@@ -30,33 +30,48 @@
 #include "images/core.pb-c.h"
 #include "images/creds.pb-c.h"
 
-#ifdef CONFIG_COMPAT
-static int has_arch_map_vdso(void)
+int kdat_can_map_vdso(void)
 {
-	unsigned long auxval;
-	int ret;
+	pid_t child;
+	int stat;
 
-	errno = 0;
-	auxval = getauxval(AT_SYSINFO_EHDR);
-	if (!auxval) {
-		if (errno == ENOENT) { /* No vDSO - OK */
-			pr_warn("No SYSINFO_EHDR - no vDSO\n");
-			return 1;
-		} else { /* That can't happen, according to man */
-			pr_err("Failed to get auxval: errno %d\n", errno);
-			return -1;
-		}
-	}
 	/*
-	 * Mapping vDSO while have not unmap it yet:
-	 * this is restricted by API if ARCH_MAP_VDSO_* is supported.
+	 * Running under fork so if vdso_64 is disabled - don't create
+	 * it for criu accidentally.
 	 */
-	ret = syscall(SYS_arch_prctl, ARCH_MAP_VDSO_32, 1);
-	if (ret == -1 && errno == EEXIST)
-		return 1;
-	return 0;
+	child = fork();
+	if (child < 0)
+		return -1;
+
+	if (child == 0) {
+		int ret;
+
+		ret = syscall(SYS_arch_prctl, ARCH_MAP_VDSO_32, 0);
+		if (ret == 0)
+			exit(1);
+		/*
+		 * Mapping vDSO while have not unmap it yet:
+		 * this is restricted by API if ARCH_MAP_VDSO_* is supported.
+		 */
+		if (ret == -1 && errno == EEXIST)
+			exit(1);
+		exit(0);
+	}
+
+	if (waitpid(child, &stat, 0) != child) {
+		pr_err("Failed to wait for arch_prctl() test\n");
+		kill(child, SIGKILL);
+		return -1;
+	}
+
+	if (!WIFEXITED(stat))
+		return -1;
+
+	return WEXITSTATUS(stat);
+
 }
 
+#ifdef CONFIG_COMPAT
 void *mmap_ia32(void *addr, size_t len, int prot,
 		int flags, int fildes, off_t off)
 {
@@ -139,7 +154,7 @@ static int has_32bit_mmap_bug(void)
 		mmap_bug_test();
 
 	if (waitpid(child, &stat, 0) != child) {
-		pr_err("Failed to wait for mmap test");
+		pr_err("Failed to wait for mmap test\n");
 		kill(child, SIGKILL);
 		return -1;
 	}
@@ -151,13 +166,15 @@ static int has_32bit_mmap_bug(void)
 
 int kdat_compatible_cr(void)
 {
-	if (!has_arch_map_vdso())
+	if (!kdat.can_map_vdso)
 		return 0;
+
 	if (has_32bit_mmap_bug())
 		return 0;
+
 	return 1;
 }
-#else
+#else /* !CONFIG_COMPAT */
 int kdat_compatible_cr(void)
 {
 	return 0;
