@@ -276,6 +276,9 @@ void tcp_cork(int sk, bool on);
 
 const char *ns_to_string(unsigned int ns);
 
+int xatol(const char *string, long *number);
+int xatoi(const char *string, int *number);
+
 char *xstrcat(char *str, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
 char *xsprintf(const char *fmt, ...)
@@ -319,7 +322,21 @@ int setup_tcp_client(char *addr);
 
 struct epoll_rfd {
 	int fd;
-	int (*revent)(struct epoll_rfd *);
+	/*
+	 * EPOLLIN notification. The data is available for read in
+	 * rfd->fd.
+	 * @return 0 to resume polling, 1 to stop polling or a
+	 * negative error code
+	 */
+	int (*read_event)(struct epoll_rfd *);
+
+	/*
+	 * EPOLLHUP | EPOLLRDHUP notification. The remote side has
+	 * close the connection for rfd->fd.
+	 * @return 0 to resume polling, 1 to stop polling or a
+	 * negative error code
+	 */
+	int (*hangup_event)(struct epoll_rfd *);
 };
 
 extern int epoll_add_rfd(int epfd, struct epoll_rfd *);
