@@ -8,11 +8,12 @@
 #include <errno.h>
 #include <sys/syscall.h>
 #include <sys/sysmacros.h>
-#include <sys/sysmacros.h>
 #include <stdint.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>  /* for sockaddr_in and inet_ntoa() */
+#include <sys/prctl.h>
 
+#include "common/config.h"
 #include "int.h"
 #include "log.h"
 #include "restorer.h"
@@ -25,9 +26,14 @@
 #include "util.h"
 #include "lsm.h"
 #include "proc_parse.h"
-#include "config.h"
-#include "syscall-codes.h"
 #include "sk-inet.h"
+#include <compel/plugins/std/syscall-codes.h>
+#include <compel/compel.h>
+#include "netfilter.h"
+#include "linux/userfaultfd.h"
+#include "prctl.h"
+#include "uffd.h"
+#include "vdso.h"
 
 struct kerndat_s kdat = {
 };
@@ -293,9 +299,8 @@ int kerndat_get_dirty_track(void)
 		goto no_dt;
 
 	ret = -1;
-	pm2 = open("/proc/self/pagemap", O_RDONLY);
+	pm2 = open_proc(PROC_SELF, "pagemap");
 	if (pm2 < 0) {
-		pr_perror("Can't open pagemap file");
 		munmap(map, PAGE_SIZE);
 		return ret;
 	}
@@ -348,7 +353,7 @@ static int init_zero_page_pfn()
 		return -1;
 	}
 
-	ret = vaddr_to_pfn((unsigned long)addr, &kdat.zero_page_pfn);
+	ret = vaddr_to_pfn(-1, (unsigned long)addr, &kdat.zero_page_pfn);
 	munmap(addr, PAGE_SIZE);
 
 	if (kdat.zero_page_pfn == 0)
@@ -386,7 +391,7 @@ static bool kerndat_has_memfd_create(void)
 
 static int get_task_size(void)
 {
-	kdat.task_size = task_size();
+	kdat.task_size = compel_task_size();
 	pr_debug("Found task size of %lx\n", kdat.task_size);
 	return 0;
 }
@@ -396,11 +401,9 @@ int kerndat_fdinfo_has_lock()
 	int fd, pfd = -1, exit_code = -1, len;
 	char buf[PAGE_SIZE];
 
-	fd = open("/proc/locks", O_RDONLY);
-	if (fd < 0) {
-		pr_perror("Unable to open /proc/locks");
+	fd = open_proc(PROC_GEN, "locks");
+	if (fd < 0)
 		return -1;
-	}
 
 	if (flock(fd, LOCK_SH)) {
 		pr_perror("Can't take a lock");
@@ -443,22 +446,19 @@ static int get_ipv6()
 	return 0;
 }
 
-int kerndat_loginuid(bool only_dump)
+int kerndat_loginuid(void)
 {
 	unsigned int saved_loginuid;
 	int ret;
 
-	kdat.has_loginuid = false;
+	kdat.luid = LUID_NONE;
 
 	/* No such file: CONFIG_AUDITSYSCALL disabled */
 	saved_loginuid = parse_pid_loginuid(PROC_SELF, &ret, true);
 	if (ret < 0)
 		return 0;
 
-	if (only_dump) {
-		kdat.has_loginuid = true;
-		return 0;
-	}
+	kdat.luid = LUID_READ;
 
 	/*
 	 * From kernel v3.13-rc2 it's possible to unset loginuid value,
@@ -471,7 +471,7 @@ int kerndat_loginuid(bool only_dump)
 	if (prepare_loginuid(saved_loginuid, LOG_WARN) < 0)
 		return 0;
 
-	kdat.has_loginuid = true;
+	kdat.luid = LUID_FULL;
 	return 0;
 }
 
@@ -557,10 +557,285 @@ err:
 	return exit_code;
 }
 
+static int kerndat_compat_restore(void)
+{
+	int ret;
+
+	ret = kdat_can_map_vdso();
+	if (ret < 0)
+		return ret;
+	kdat.can_map_vdso = !!ret;
+
+	/* depends on kdat.can_map_vdso result */
+	kdat.compat_cr = kdat_compatible_cr();
+
+	return 0;
+}
+
+static int kerndat_detect_stack_guard_gap(void)
+{
+	int num, ret = -1, detected = 0;
+	unsigned long start, end;
+	char r, w, x, s;
+	char buf[1024];
+	FILE *maps;
+	void *mem;
+
+	mem = mmap(NULL, (3ul << 20), PROT_READ | PROT_WRITE,
+		   MAP_PRIVATE | MAP_ANONYMOUS | MAP_GROWSDOWN, -1, 0);
+	if (mem == MAP_FAILED) {
+		pr_perror("Can't mmap stack area");
+		return -1;
+	}
+	munmap(mem, (3ul << 20));
+
+	mem = mmap(mem + (2ul << 20), (1ul << 20), PROT_READ | PROT_WRITE,
+		   MAP_PRIVATE | MAP_ANONYMOUS | MAP_FIXED | MAP_GROWSDOWN, -1, 0);
+	if (mem == MAP_FAILED) {
+		pr_perror("Can't mmap stack area");
+		return -1;
+	}
+
+	maps = fopen("/proc/self/maps", "r");
+	if (maps == NULL) {
+		munmap(mem, 4096);
+		return -1;
+	}
+
+	while (fgets(buf, sizeof(buf), maps)) {
+		num = sscanf(buf, "%lx-%lx %c%c%c%c",
+			     &start, &end, &r, &w, &x, &s);
+		if (num < 6) {
+			pr_err("Can't parse: %s\n", buf);
+			goto err;
+		}
+
+		/*
+		 * When reading /proc/$pid/[s]maps the
+		 * start/end addresses migh be cutted off
+		 * with PAGE_SIZE on kernels prior 4.12
+		 * (see kernel commit 1be7107fbe18ee).
+		 *
+		 * Same time there was semi-complete
+		 * patch released which hitted a number
+		 * of repos (Ubuntu, Fedora) where instead
+		 * of PAGE_SIZE the 1M gap is cutted off.
+		 */
+		if (start == (unsigned long)mem) {
+			kdat.stack_guard_gap_hidden = false;
+			detected = 1;
+			break;
+		} else if (start == ((unsigned long)mem + (1ul << 20))) {
+			pr_warn("Unsupported stack guard detected, confused but continue\n");
+			kdat.stack_guard_gap_hidden = true;
+			detected = 1;
+			break;
+		} else if (start == ((unsigned long)mem + PAGE_SIZE)) {
+			kdat.stack_guard_gap_hidden = true;
+			detected = 1;
+			break;
+		}
+	}
+
+	if (detected)
+		ret = 0;
+
+err:
+	munmap(mem, (1ul << 20));
+	fclose(maps);
+	return ret;
+}
+
+#define KERNDAT_CACHE_FILE	KDAT_RUNDIR"/criu.kdat"
+#define KERNDAT_CACHE_FILE_TMP	KDAT_RUNDIR"/.criu.kdat"
+
+static int kerndat_try_load_cache(void)
+{
+	int fd, ret;
+
+	fd = open(KERNDAT_CACHE_FILE, O_RDONLY);
+	if (fd < 0) {
+		pr_warn("Can't load %s\n", KERNDAT_CACHE_FILE);
+		return 1;
+	}
+
+	ret = read(fd, &kdat, sizeof(kdat));
+	if (ret < 0) {
+		pr_perror("Can't read kdat cache");
+		return -1;
+	}
+
+	close(fd);
+
+	if (ret != sizeof(kdat) ||
+			kdat.magic1 != KDAT_MAGIC ||
+			kdat.magic2 != KDAT_MAGIC_2) {
+		pr_warn("Stale %s file\n", KERNDAT_CACHE_FILE);
+		unlink(KERNDAT_CACHE_FILE);
+		return 1;
+	}
+
+	pr_info("Loaded kdat cache from %s\n", KERNDAT_CACHE_FILE);
+	return 0;
+}
+
+static void kerndat_save_cache(void)
+{
+	int fd, ret;
+	struct statfs s;
+
+	fd = open(KERNDAT_CACHE_FILE_TMP, O_CREAT | O_EXCL | O_WRONLY, 0600);
+	if (fd < 0)
+		/*
+		 * It can happen that we race with some other criu
+		 * instance. That's OK, just ignore this error and
+		 * proceed.
+		 */
+		return;
+
+	if (fstatfs(fd, &s) < 0 || s.f_type != TMPFS_MAGIC) {
+		pr_warn("Can't keep kdat cache on non-tempfs\n");
+		close(fd);
+		goto unl;
+	}
+
+	/*
+	 * One magic to make sure we're reading the kdat file.
+	 * One more magic to make somehow sure we don't read kdat
+	 * from some other criu
+	 */
+	kdat.magic1 = KDAT_MAGIC;
+	kdat.magic2 = KDAT_MAGIC_2;
+	ret = write(fd, &kdat, sizeof(kdat));
+	close(fd);
+
+	if (ret == sizeof(kdat))
+		ret = rename(KERNDAT_CACHE_FILE_TMP, KERNDAT_CACHE_FILE);
+	else {
+		ret = -1;
+		errno = EIO;
+	}
+
+	if (ret < 0) {
+		pr_perror("Couldn't save %s", KERNDAT_CACHE_FILE);
+unl:
+		unlink(KERNDAT_CACHE_FILE_TMP);
+	}
+}
+
+int kerndat_uffd(void)
+{
+	int uffd;
+
+	kdat.uffd_features = 0;
+	uffd = uffd_open(0, &kdat.uffd_features);
+
+	/*
+	 * uffd == -ENOSYS means userfaultfd is not supported on this
+	 * system and we just happily return with kdat.has_uffd = false.
+	 * Error other than -ENOSYS would mean "Houston, Houston, we
+	 * have a problem!"
+	 */
+	if (uffd < 0) {
+		if (uffd == -ENOSYS)
+			return 0;
+
+		pr_err("Lazy pages are not available\n");
+		return -1;
+	}
+
+	kdat.has_uffd = true;
+
+	/*
+	 * we have to close the uffd and reopen in later in restorer
+	 * to enable non-cooperative features
+	 */
+	close(uffd);
+
+	return 0;
+}
+
+int kerndat_has_thp_disable(void)
+{
+	struct bfd f;
+	void *addr;
+	char *str;
+	int ret = -1;
+	bool vma_match = false;
+
+	if (prctl(PR_SET_THP_DISABLE, 1, 0, 0, 0)) {
+		if (errno != EINVAL)
+			return -1;
+		pr_info("PR_SET_THP_DISABLE is not available\n");
+		return 0;
+	}
+
+	addr = mmap(NULL, PAGE_SIZE, PROT_READ | PROT_WRITE,
+		    MAP_PRIVATE | MAP_ANONYMOUS, 0, 0);
+	if (addr == MAP_FAILED) {
+		pr_perror("Can't mmap memory for THP disable test");
+		return -1;
+	}
+
+	if (prctl(PR_SET_THP_DISABLE, 0, 0, 0, 0))
+		return -1;
+
+	f.fd = open("/proc/self/smaps", O_RDONLY);
+	if (f.fd < 0) {
+		pr_perror("Can't open /proc/self/smaps");
+		goto out_unmap;
+	}
+	if (bfdopenr(&f))
+		goto out_unmap;
+
+	while ((str = breadline(&f)) != NULL) {
+		if (IS_ERR(str))
+			goto out_close;
+
+		if (is_vma_range_fmt(str)) {
+			unsigned long vma_addr;
+
+			if (sscanf(str, "%lx-", &vma_addr) != 1) {
+				pr_err("Can't parse: %s\n", str);
+				goto out_close;
+			}
+
+			if (vma_addr == (unsigned long)addr)
+				vma_match = true;
+		}
+
+		if (vma_match && !strncmp(str, "VmFlags: ", 9)) {
+			u32 flags = 0;
+			u64 madv = 0;
+			int io_pf = 0;
+
+			parse_vmflags(str, &flags, &madv, &io_pf);
+			kdat.has_thp_disable = !(madv & (1 << MADV_NOHUGEPAGE));
+			break;
+		}
+	}
+
+	ret = 0;
+
+out_close:
+	bclose(&f);
+out_unmap:
+	munmap(addr, PAGE_SIZE);
+
+	return ret;
+}
+
 int kerndat_init(void)
 {
 	int ret;
 
+	ret = kerndat_try_load_cache();
+	if (ret <= 0)
+		return ret;
+
+	preload_socket_modules();
+	preload_netfilter_modules();
+
 	ret = check_pagemap();
 	if (!ret)
 		ret = kerndat_get_shmemdev();
@@ -577,55 +852,33 @@ int kerndat_init(void)
 	if (!ret)
 		ret = get_ipv6();
 	if (!ret)
-		ret = kerndat_loginuid(true);
+		ret = kerndat_loginuid();
 	if (!ret)
 		ret = kerndat_iptables_has_xtlocks();
 	if (!ret)
 		ret = kerndat_tcp_repair();
-
-	kerndat_lsm();
-	kerndat_mmap_min_addr();
-
-	return ret;
-}
-
-int kerndat_init_rst(void)
-{
-	int ret;
-
-	/*
-	 * Read TCP sysctls before anything else,
-	 * since the limits we're interested in are
-	 * not available inside namespaces.
-	 */
-
-	ret = check_pagemap();
 	if (!ret)
-		ret = get_last_cap();
+		ret = kerndat_compat_restore();
 	if (!ret)
 		ret = kerndat_has_memfd_create();
 	if (!ret)
-		ret = get_task_size();
+		ret = kerndat_detect_stack_guard_gap();
 	if (!ret)
-		ret = get_ipv6();
+		ret = kerndat_uffd();
 	if (!ret)
-		ret = kerndat_loginuid(false);
+		ret = kerndat_has_thp_disable();
+	/* Needs kdat.compat_cr filled before */
 	if (!ret)
-		ret = kerndat_iptables_has_xtlocks();
+		ret = kerndat_vdso_fill_symtable();
+	/* Depends on kerndat_vdso_fill_symtable() */
 	if (!ret)
-		ret = kerndat_tcp_repair();
+		ret = kerndat_vdso_preserves_hint();
 
 	kerndat_lsm();
 	kerndat_mmap_min_addr();
 
-	return ret;
-}
-
-int kerndat_init_cr_exec(void)
-{
-	int ret;
-
-	ret = get_task_size();
+	if (!ret)
+		kerndat_save_cache();
 
 	return ret;
 }
