@@ -8,6 +8,8 @@
 
 #include "../soccr/soccr.h"
 
+#include "common/config.h"
+#include "cr_options.h"
 #include "util.h"
 #include "common/list.h"
 #include "log.h"
@@ -18,7 +20,6 @@
 #include "image.h"
 #include "namespaces.h"
 #include "xmalloc.h"
-#include "config.h"
 #include "kerndat.h"
 #include "restorer.h"
 #include "rst-malloc.h"
@@ -112,8 +113,10 @@ static int dump_tcp_conn_state(struct inet_sk_desc *sk)
 	struct libsoccr_sk_data data;
 
 	ret = libsoccr_save(socr, &data, sizeof(data));
-	if (ret < 0)
+	if (ret < 0) {
+		pr_err("libsoccr_save() failed with %d\n", ret);
 		goto err_r;
+	}
 	if (ret != sizeof(data)) {
 		pr_err("This libsocr is not supported (%d vs %d)\n",
 				ret, (int)sizeof(data));
@@ -407,6 +410,11 @@ int restore_one_tcp(int fd, struct inet_sk_info *ii)
 
 	pr_info("Restoring TCP connection\n");
 
+	if (opts.tcp_close &&
+		ii->ie->state != TCP_LISTEN && ii->ie->state != TCP_CLOSE) {
+		return 0;
+	}
+
 	sk = libsoccr_pause(fd);
 	if (!sk)
 		return -1;
