@@ -524,6 +524,10 @@ int dump_socket_opts(int sk, SkOptsEntry *soe)
 	soe->reuseaddr = val ? true : false;
 	soe->has_reuseaddr = true;
 
+	ret |= dump_opt(sk, SOL_SOCKET, SO_REUSEPORT, &val);
+	soe->so_reuseport = val ? true : false;
+	soe->has_so_reuseport = true;
+
 	ret |= dump_opt(sk, SOL_SOCKET, SO_PASSCRED, &val);
 	soe->has_so_passcred = true;
 	soe->so_passcred = val ? true : false;
@@ -552,7 +556,7 @@ void release_skopts(SkOptsEntry *soe)
 	xfree(soe->so_bound_dev);
 }
 
-int dump_socket(struct fd_parms *p, int lfd, struct cr_img *img)
+int dump_socket(struct fd_parms *p, int lfd, FdinfoEntry *e)
 {
 	int family;
 	const struct fdtype_ops *ops;
@@ -581,7 +585,7 @@ int dump_socket(struct fd_parms *p, int lfd, struct cr_img *img)
 		return -1;
 	}
 
-	return do_dump_gen_file(p, lfd, ops, img);
+	return do_dump_gen_file(p, lfd, ops, e);
 }
 
 static int inet_receive_one(struct nlmsghdr *h, void *arg)
