@@ -56,6 +56,8 @@
 
 #define LAZY_PAGES_SOCK_NAME	"lazy-pages.socket"
 
+#define LAZY_PAGES_RESTORE_FINISHED	0x52535446	/* ReSTore Finished */
+
 static mutex_t *lazy_sock_mutex;
 
 struct lazy_iov {
@@ -99,6 +101,8 @@ static LIST_HEAD(lpis);
 static LIST_HEAD(exiting_lpis);
 static LIST_HEAD(pending_lpis);
 static int epollfd;
+static bool restore_finished;
+static struct epoll_rfd lazy_sk_rfd;
 
 static int handle_uffd_event(struct epoll_rfd *lpfd);
 
@@ -114,7 +118,7 @@ static struct lazy_pages_info *lpi_init(void)
 	INIT_LIST_HEAD(&lpi->iovs);
 	INIT_LIST_HEAD(&lpi->reqs);
 	INIT_LIST_HEAD(&lpi->l);
-	lpi->lpfd.revent = handle_uffd_event;
+	lpi->lpfd.read_event = handle_uffd_event;
 
 	return lpi;
 }
@@ -460,6 +464,8 @@ static int drop_iovs(struct lazy_pages_info *lpi, unsigned long addr, int len)
 			addr = start;
 		}
 
+		iov->queued = false;
+
 		/*
 		 * The range completely fits into the current IOV.
 		 * If addr equals iov_base we just "drop" the
@@ -705,6 +711,27 @@ static int handle_exit(struct lazy_pages_info *lpi)
 	return 0;
 }
 
+static int uffd_check_op_error(struct lazy_pages_info *lpi, const char *op,
+			       unsigned long len, unsigned long rc_len, int rc)
+{
+	if (rc) {
+		if (errno == ENOSPC || errno == ESRCH) {
+			handle_exit(lpi);
+			return 0;
+		}
+		if (rc_len != -EEXIST) {
+			lp_perror(lpi, "%s: rc:%d copy:%ld, errno:%d",
+				 op, rc, rc_len, errno);
+			return -1;
+		}
+	} else if (rc_len != len) {
+		lp_err(lpi, "%s unexpected size %ld\n", op, rc_len);
+		return -1;
+	}
+
+	return 0;
+}
+
 static int uffd_copy(struct lazy_pages_info *lpi, __u64 address, int nr_pages)
 {
 	struct uffdio_copy uffdio_copy;
@@ -719,20 +746,8 @@ static int uffd_copy(struct lazy_pages_info *lpi, __u64 address, int nr_pages)
 
 	lp_debug(lpi, "uffd_copy: 0x%llx/%ld\n", uffdio_copy.dst, len);
 	rc = ioctl(lpi->lpfd.fd, UFFDIO_COPY, &uffdio_copy);
-	if (rc) {
-		if (errno == ENOSPC || errno == ESRCH) {
-			handle_exit(lpi);
-			return 0;
-		}
-		if (uffdio_copy.copy != -EEXIST) {
-			lp_debug(lpi, "uffd_copy: rc:%d copy:%Ld, errno:%d\n",
-				 rc, uffdio_copy.copy, errno);
-			return -1;
-		}
-	} else if (uffdio_copy.copy != len) {
-		lp_err(lpi, "UFFDIO_COPY unexpected size %Ld\n", uffdio_copy.copy);
+	if (uffd_check_op_error(lpi, "copy", len, uffdio_copy.copy, rc))
 		return -1;
-	}
 
 	lpi->copied_pages += nr_pages;
 
@@ -784,11 +799,8 @@ static int uffd_zero(struct lazy_pages_info *lpi, __u64 address, int nr_pages)
 
 	lp_debug(lpi, "zero page at 0x%llx\n", address);
 	rc = ioctl(lpi->lpfd.fd, UFFDIO_ZEROPAGE, &uffdio_zeropage);
-	if (rc) {
-		lp_perror(lpi, "zero page failed: %Ld",
-			  uffdio_zeropage.zeropage);
+	if (uffd_check_op_error(lpi, "zero", len, uffdio_zeropage.zeropage, rc))
 		return -1;
-	}
 
 	return 0;
 }
@@ -908,6 +920,15 @@ static int handle_remove(struct lazy_pages_info *lpi, struct uffd_msg *msg)
 	 */
 	if (msg->event == UFFD_EVENT_REMOVE &&
 	    ioctl(lpi->lpfd.fd, UFFDIO_UNREGISTER, &unreg)) {
+		/*
+		 * The kernel returns -ENOMEM when unregister is
+		 * called after the process has gone
+		 */
+		if (errno == ENOMEM) {
+			handle_exit(lpi);
+			return 0;
+		}
+
 		pr_perror("Failed to unregister (%llx - %llx)", unreg.start,
 			  unreg.start + unreg.len);
 		return -1;
@@ -1088,6 +1109,7 @@ static void lazy_pages_summary(struct lazy_pages_info *lpi)
 static int handle_requests(int epollfd, struct epoll_event *events, int nr_fds)
 {
 	struct lazy_pages_info *lpi, *n;
+	/* FIXME -- timeout should decrease over time...  */
 	int poll_timeout = POLL_TIMEOUT;
 	int ret;
 
@@ -1103,6 +1125,10 @@ static int handle_requests(int epollfd, struct epoll_event *events, int nr_fds)
 			continue;
 		}
 
+		/* don't start backround fetch before restore is finished */
+		if (!restore_finished)
+			continue;
+
 		if (poll_timeout)
 			pr_debug("Start handling remaining pages\n");
 
@@ -1133,6 +1159,29 @@ out:
 
 }
 
+int lazy_pages_finish_restore(void)
+{
+	uint32_t fin = LAZY_PAGES_RESTORE_FINISHED;
+	int fd, ret;
+
+	if (!opts.lazy_pages)
+		return 0;
+
+	fd = get_service_fd(LAZY_PAGES_SK_OFF);
+	if (fd < 0) {
+		pr_err("No lazy-pages socket\n");
+		return -1;
+	}
+
+	ret = send(fd, &fin, sizeof(fin), 0);
+	if (ret != sizeof(fin))
+		pr_perror("Failed sending restore finished indication");
+
+	close(fd);
+
+	return ret < 0 ? ret : 0;
+}
+
 static int prepare_lazy_socket(void)
 {
 	int listen;
@@ -1150,6 +1199,44 @@ static int prepare_lazy_socket(void)
 	return listen;
 }
 
+static int lazy_sk_read_event(struct epoll_rfd *rfd)
+{
+	uint32_t fin;
+	int ret;
+
+	ret = recv(rfd->fd, &fin, sizeof(fin), 0);
+	/*
+	 * epoll sets POLLIN | POLLHUP for the EOF case, so we get short
+	 * read just befor hangup_event
+	 */
+	if (!ret)
+		return 0;
+
+	if (ret != sizeof(fin)) {
+		pr_perror("Failed getting restore finished inidication");
+		return -1;
+	}
+
+	if (fin != LAZY_PAGES_RESTORE_FINISHED) {
+		pr_err("Unexpected response: %x\n", fin);
+		return -1;
+	}
+
+	restore_finished = true;
+
+	return 0;
+}
+
+static int lazy_sk_hangup_event(struct epoll_rfd *rfd)
+{
+	if (!restore_finished) {
+		pr_err("Restorer unexpectedly closed the connection\n");
+		return -1;
+	}
+
+	return 0;
+}
+
 static int prepare_uffds(int listen, int epollfd)
 {
 	int i;
@@ -1175,7 +1262,12 @@ static int prepare_uffds(int listen, int epollfd)
 			goto close_uffd;
 	}
 
-	close_safe(&client);
+	lazy_sk_rfd.fd = client;
+	lazy_sk_rfd.read_event = lazy_sk_read_event;
+	lazy_sk_rfd.hangup_event = lazy_sk_hangup_event;
+	if (epoll_add_rfd(epollfd, &lazy_sk_rfd))
+		goto close_uffd;
+
 	close(listen);
 	return 0;
 
@@ -1225,7 +1317,12 @@ int cr_lazy_pages(bool daemon)
 	if (close_status_fd())
 		return -1;
 
-	nr_fds = task_entries->nr_tasks + (opts.use_page_server ? 1 : 0);
+	/*
+	 * we poll nr_tasks userfault fds, UNIX socket between lazy-pages
+	 * daemon and the cr-restore, and, optionally TCP socket for
+	 * remote pages
+	 */
+	nr_fds = task_entries->nr_tasks + (opts.use_page_server ? 2 : 1);
 	epollfd = epoll_prepare(nr_fds, &events);
 	if (epollfd < 0)
 		return -1;
