@@ -3,6 +3,7 @@
 #include <sys/socket.h>
 #include <netinet/tcp.h>
 #include <netinet/in.h>
+#include <linux/types.h>
 
 #include "zdtmtst.h"
 
@@ -14,7 +15,9 @@ static int port = 8880;
 static int check_socket_closed(int sk)
 {
 	int err, buffer = 0;
-	struct tcp_info info;
+	struct {
+		__u8    tcpi_state;
+	} info;
 	socklen_t len = sizeof(info);
 
 	err = getsockopt(sk, IPPROTO_TCP, TCP_INFO, (void *)&info, &len);
